@@ -18,13 +18,8 @@ from . import chars
 class Cell:
     def __init__(self, solution, entry=None):
         self.solution = solution
-
         self.number = None
-        if entry:
-            self.entry = entry
-        else:
-            self.entry = "-"
-
+        self.entry = entry or "-"
         self.marked_wrong = False
         self.corrected = False
         self.revealed = False
@@ -115,6 +110,13 @@ class Grid:
         self.across_clues = [word['clue'] for word in num.across]
         self.down_clues = [word['clue'] for word in num.down]
 
+        self.across_spaces = [(j,i) for i in range(self.column_count)
+                                    for j in range(self.row_count)
+                                    if self.cells[(j,i)].is_letter()]
+        self.down_spaces = [(j,i) for j in range(self.row_count)
+                                  for i in range(self.column_count)
+                                  if self.cells[(j,i)].is_letter()]
+
         if self.puzfile.has_markup():
             markup = self.puzfile.markup().markup
 
@@ -138,8 +140,6 @@ class Grid:
         else:
             self.start_time, self.timer_active = 0, 1
 
-        return None
-
     def draw(self):
         top_row = self.get_top_row()
         bottom_row = self.get_bottom_row()
@@ -154,15 +154,13 @@ class Grid:
                                 1):
             if index % 2 == 0:
                 print(self.term.move(y_val, self.grid_x) +
-                        self.term.dim(divider_row))
+                      self.term.dim(divider_row))
             else:
                 print(self.term.move(y_val, self.grid_x) +
-                        self.term.dim(middle_row))
+                      self.term.dim(middle_row))
         print(self.term.move(self.grid_y + self.row_count * 2, self.grid_x)
               + self.term.dim(bottom_row))
 
-        return None
-
     def number(self):
         numbered_squares = []
         for word in self.across_words:
@@ -177,8 +175,6 @@ class Grid:
         for number, square in enumerate(numbered_squares, 1):
             self.cells.get(square).number = number
 
-        return None
-
     def fill(self):
         for position in self.cells:
             y_coord, x_coord = self.to_term(position)
@@ -190,42 +186,27 @@ class Grid:
                         self.term.dim(chars.squareblock))
 
             if cell.number:
-                small = small_nums(cell.number)
+                small = str(cell.number).translate(chars.small_nums)
                 x_pos = x_coord - 1
                 print(self.term.move(y_coord - 1, x_pos) + small)
 
-        return None
-
     def confirm_quit(self, modified_since_save):
-        confirmed = True
         if modified_since_save:
             confirmation = self.get_notification_input(
                                 "Quit without saving? (y/n)",
                                 chars=1, blocking=True, timeout=5)
-            if confirmation.lower() == 'y':
-                confirmed = True
-            else:
-                confirmed = False
-
-        return confirmed
+            return confirmation.lower() == 'y'
+        return True
 
     def confirm_clear(self):
         confirmation = self.get_notification_input("Clear puzzle? (y/n)",
                                 chars=1, blocking=True, timeout=5)
-        if confirmation.lower() == 'y':
-            confirmed = True
-        else:
-            confirmed = False
-        return confirmed
+        return confirmation.lower() == 'y'
 
     def confirm_reset(self):
         confirmation = self.get_notification_input("Reset puzzle? (y/n)",
                                 chars=1, blocking=True, timeout=5)
-        if confirmation.lower() == 'y':
-            confirmed = True
-        else:
-            confirmed = False
-        return confirmed
+        return confirmation.lower() == 'y'
 
     def save(self, filename):
         fill = ''
@@ -257,11 +238,9 @@ class Grid:
                 if cell.circled:
                     cell_md += 128
                 md.append(cell_md)
-
             self.puzfile.markup().markup = md
 
         self.puzfile.save(filename)
-
         self.send_notification("Current puzzle state saved.")
 
     def reveal_cell(self, pos):
@@ -293,12 +272,10 @@ class Grid:
 
 
     def make_row(self, leftmost, middle, divider, rightmost):
-        row = leftmost
+        chars = ''
         for col in range(1, self.column_count * 4):
-            new_char = divider if col % 4 == 0 else middle
-            row += new_char
-        row += rightmost
-        return row
+            chars += divider if col % 4 == 0 else middle
+        return leftmost + chars + rightmost
 
     def get_top_row(self):
         return self.make_row(chars.ulcorner, chars.hline, chars.ttee, chars.urcorner)
@@ -314,15 +291,12 @@ class Grid:
 
     def compile_cell(self, position):
         cell = self.cells.get(position)
-        if cell.is_blank():
-            value = " "
-        else:
-            value = cell.entry
+        value = " " if cell.is_blank() else cell.entry
 
         if cell.circled:
-            value = encircle(value)
+            value = value.translate(chars.encircle)
 
-        if cell.marked_wrong:
+        if cell.marked_wrong and not cell.revealed:
             value = self.term.red(value.lower())
         else:
             value = self.term.bold(value)
@@ -477,6 +451,9 @@ class Cursor:
             word_group = self.grid.down_words_grouped
             next_words = self.grid.across_words
 
+        while self.current_word() not in word_group:
+            self.retreat()
+
         word_index = word_group.index(self.current_word())
 
         if word_index == len(word_group) - 1:
@@ -491,7 +468,7 @@ class Cursor:
         # the blank_placement setting
         if (blank_placement and
                 not any(self.grid.cells.get(pos).is_blankish() for
-                pos in itertools.chain(*self.grid.across_words))):
+                pos in self.grid.across_spaces + self.grid.down_spaces)):
             blank_placement = False
 
         # Otherwise, if blank_placement is on, put the
@@ -512,6 +489,9 @@ class Cursor:
             word_group = self.grid.down_words_grouped
             next_words = self.grid.across_words
 
+        while self.current_word() not in word_group:
+            self.advance()
+
         word_index = word_group.index(self.current_word())
 
         pos = -1 if end_placement else 0
@@ -527,7 +507,7 @@ class Cursor:
         # the blank_placement setting
         if (blank_placement and
                 not any(self.grid.cells.get(pos).is_blankish() for
-                pos in itertools.chain(*self.grid.across_words))):
+                pos in self.grid.across_spaces + self.grid.down_spaces)):
             blank_placement = False
 
         if blank_placement and self.earliest_blank_in_word():
@@ -541,14 +521,14 @@ class Cursor:
         return next(blanks, None)
 
     def move_right(self):
-        spaces = list(itertools.chain(*self.grid.across_words))
+        spaces = self.grid.across_spaces
         current_space = spaces.index(self.position)
         ordered_spaces = spaces[current_space + 1:] + spaces[:current_space]
 
         return next(iter(ordered_spaces))
 
     def move_left(self):
-        spaces = list(itertools.chain(*self.grid.across_words))
+        spaces = self.grid.across_spaces
         current_space = spaces.index(self.position)
         ordered_spaces = (spaces[current_space - 1::-1] +
                           spaces[:current_space:-1])
@@ -556,14 +536,14 @@ class Cursor:
         return next(iter(ordered_spaces))
 
     def move_down(self):
-        spaces = list(itertools.chain(*self.grid.down_words))
+        spaces = self.grid.down_spaces
         current_space = spaces.index(self.position)
         ordered_spaces = spaces[current_space + 1:] + spaces[:current_space]
 
         return next(iter(ordered_spaces))
 
     def move_up(self):
-        spaces = list(itertools.chain(*self.grid.down_words))
+        spaces = self.grid.down_spaces
         current_space = spaces.index(self.position)
         ordered_spaces = (spaces[current_space - 1::-1] +
                           spaces[:current_space:-1])
@@ -575,9 +555,9 @@ class Cursor:
         word = []
 
         if self.direction == "across":
-            word = next((w for w in self.grid.across_words if pos in w), [])
+            word = next((w for w in self.grid.across_words if pos in w), [pos])
         if self.direction == "down":
-            word = next((w for w in self.grid.down_words if pos in w), [])
+            word = next((w for w in self.grid.down_words if pos in w), [pos])
 
         return word
 
@@ -659,24 +639,6 @@ class Timer(threading.Thread):
         self.is_running = True
 
 
-def small_nums(number):
-    small_num = ""
-    num_dict = {"1": "₁", "2": "₂", "3": "₃", "4": "₄", "5": "₅",
-                "6": "₆", "7": "₇", "8": "₈", "9": "₉", "0": "₀"}
-    for digit in str(number):
-        small_num += num_dict[digit]
-
-    return small_num
-
-def encircle(letter):
-    circle_dict = {"A": "Ⓐ", "B": "Ⓑ", "C": "Ⓒ", "D": "Ⓓ", "E": "Ⓔ", "F": "Ⓕ",
-                   "G": "Ⓖ", "H": "Ⓗ", "I": "Ⓘ", "J": "Ⓙ", "K": "Ⓚ", "L": "Ⓛ",
-                   "M": "Ⓜ", "N": "Ⓝ", "O": "Ⓞ", "P": "Ⓟ", "Q": "Ⓠ", "R": "Ⓡ",
-                   "S": "Ⓢ", "T": "Ⓣ", "U": "Ⓤ", "V": "Ⓥ", "W": "Ⓦ", "X": "Ⓧ",
-                   "Y": "Ⓨ", "Z": "Ⓩ", " ": "◯"}
-    return circle_dict[letter]
-
-
 def main():
     version_dir = os.path.abspath(os.path.dirname((__file__)))
     version_file = os.path.join(version_dir, 'version')
@@ -835,19 +797,25 @@ def main():
 
             # Draw the clue for the new word:
                 if cursor.direction == "across":
-                    num_index = grid.across_words.index(
+                    if cursor.current_word() in grid.across_words:
+                        num_index = grid.across_words.index(
                             cursor.current_word())
-                    clue = grid.across_clues[num_index]
-                    if downs_only:
-                        clue = "—"
+                        clue = grid.across_clues[num_index]
+                        if downs_only:
+                            clue = "—"
+                    else:
+                        clue = ""
                 elif cursor.direction == "down":
-                    num_index = grid.down_words_grouped.index(
+                    if cursor.current_word() in grid.down_words:
+                        num_index = grid.down_words_grouped.index(
                             cursor.current_word())
-                    clue = grid.down_clues[num_index]
+                        clue = grid.down_clues[num_index]
+                    else:
+                        clue = ""
 
-                num = str(grid.cells.get(cursor.current_word()[0]).number)
+                num = str(grid.cells.get(cursor.current_word()[0]).number) if clue else ""
                 compiled_clue = (num + " " + cursor.direction.upper()
-                                + ": " + clue)
+                                + ": " + clue) if num else ""
                 wrapped_clue = clue_wrapper.wrap(compiled_clue)
                 wrapped_clue += [''] * (3 - len(wrapped_clue))
                 wrapped_clue = [line + term.clear_eol for line in wrapped_clue]
@@ -1074,13 +1042,13 @@ def main():
 
                 cursor.retreat()
 
-            elif keypress in ['}', ']']:
+            elif keypress in ['}', ']'] or keypress.name in ['KEY_SRIGHT']:
                 cursor.advance_perpendicular()
                 if (keypress == '}' and blank_cells_remaining):
                     while not grid.cells.get(cursor.position).is_blankish():
                         cursor.advance_perpendicular()
 
-            elif keypress in ['{', '[']:
+            elif keypress in ['{', '['] or keypress.name in ['KEY_SLEFT']:
                 cursor.retreat_perpendicular()
                 if (keypress == '{' and blank_cells_remaining):
                     while not grid.cells.get(cursor.position).is_blankish():
